# test ipython notebook nb format 4

Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.

# Loading a notebook nbformat = 4.0

Il s’agit de repérer les données nécessaires voire indispensables à la résolution. Ces données peuvent être:

- numériques,
- ou sous forme de textes (on dit souvent chaînes de caractères),
- ou de type logique (à deux valeurs possibles, vrai ou faux).

Pour affecter une valeur à une variable en python, la syntaxe est de la forme:

    nom = valeur

Affectation
  Dans un programme, une affectation donne une valeur à une variable, elle est de la forme `v = e` avec `v` une variable et `e` une expression.

# Valeurs numériques

Python distingue les entiers(integers), des nombres à virgule(floating-point) par l'ajout d'une virgule:

In [2]:

    a = 1
    b = 1.0

    print(a, type(a), b, type(b))

    1 <class 'int'> 1.0 <class 'float'>

Mais il implémente aussi des nombres plus exotiques tels que les décimaux, les fractions ou encore les nombres complexes

# Chaînes de caractères

En python3, il existe deux grands types de chaînes de caractères, les strings, et les bytes. Les strings sont simplement entourées par des guillemets simples ou doubles.

In [3]:

    a = 'Mon texte'
    b = "Mon deuxième texte"

    print(a, type(a), b, type(b))

    Mon texte <class 'str'> Mon deuxième texte <class 'str'>

Les bytes sont précédées de la lettre b, il s'agit de texte brut utilisé par la machine mais pas toujours lisible par un humain.

In [4]:

    a = b'Mon texte'
    print(a, type(a))

    b'Mon texte' <class 'bytes'>

On ne peut représenter simplement que certains caractères(les caractères ASCII), les caractères accentués ne peuvent être écrits directement.

In [5]:

    b = b'Mon deuxième texte'

      File "<ipython-input-5-b0bf0ef2715d>", line 1
        b = b'Mon deuxième texte'
           ^
    SyntaxError: bytes can only contain ASCII literal characters.

Un encodage autre que le ASCII doit être utilisé pour ces caractères, comme par exemple le UTF-8 qui est un codage sur huit bits très répandu.

In [6]:

    b = b'Mon deuxi\xc3\xa8me texte'
    print(b)
    print(b.decode('utf-8'))

    b'Mon deuxi\xc3\xa8me texte'
    Mon deuxième texte

# Valeurs logiques

On les appelle les booléens, il ne peuvent avoir que deux valeurs:

- VRAI ou FAUX: True ou False en anglais;
- 1 ou 0.

On les utilise pour réaliser des tests.

In [7]:

    1 < 2, 1 > 2, 2 == 10, 2 >= 1.9, 2 == 2, 2 != 2

Out[7]:

    (True, False, False, True, True, False)

On peut aussi les utiliser pour savoir si une variable existe.

In [8]:

    bool(a)

Out[8]:

    True

On peut aussi combiner plusieurs tests avec les opérateurs booléens `and`, `or` et `not`.

In [9]:

    1 < 2 or 2 < 1

Out[9]:

    True

In [10]:

    1 < 2 and 2 < 1

Out[10]:

    False

In [11]:

    not(1 < 2 and 2 < 1)

Out[11]:

    True

# Listes et dictionnaires

Souvent les données pertinentes doivent être agencées sous une forme plus vaste, comme par exemple des listes où on peut ranger de façon ordonnée plusieurs valeurs.

## Listes

Les listes sont des collections ordonnées de valeurs, elles sont entourées par des crochets `[]`, leurs éléments sont séparés par des virgules.

In [12]:

    a = [ 1, 'deux' , 3]
    type(a)

Out[12]:

    list

On peut facilement accéder à la longueur de la liste grace à la fonction `len` et à chacun de ces éléments grâce à leur index (Attention le premier élement à l'index 0)

In [13]:

    len(a)

Out[13]:

    3

In [14]:

    a[0], a[2]

Out[14]:

    (1, 3)

On peut inversement connaître l'indice correspondant à une valeur grâce à l'attribut `index`.

In [15]:

    a.index(1), a.index('deux'), a.index(3)

Out[15]:

    (0, 1, 2)

## Dictionnaires

Dans un dictionnaire les valeurs de la collection ne sont pas repéré par un index, mais par une clé. Les dictionnaires sont entourés d'accolades `{}`.

In [16]:

    a = { 'nom': 'Doe' , 'prenom': 'John', 'age': 77 }
    type(a)

Out[16]:

    dict

Pour accéder aux éléments du dictionnaire, il suffit d'appeler la clé correspondante, d'autres part la fonction `len` est égalemnt disponible.

In [17]:

    len(a)

Out[17]:

    3

In [18]:

    a['nom'], a['age']

Out[18]:

    ('Doe', 77)

## Modification des listes et dictionnaires

Les listes et dictionnaires sont des objets mutables, c'est à dire que l'on peut modifier leur contenu sans créer un nouvel objet. On dit qu'il s'agit de données non-persistantes.

In [19]:

    # Valeurs initiales
    liste = [ 1, 'deux' , 3]
    dict = { 'nom': 'Doe' , 'prenom': 'John', 'age': 77 }
    print("Valeurs initiales:\n", liste, dict)

    # Modification des valeurs par assignation
    liste[1] = 2
    dict['age'] = 17

    print("Modification des valeurs par assignation:\n", liste, dict)

    # Ajout d'éléments
    liste.append(4)
    dict['nationalité'] = 'française'

    print("Ajout d'éléments:\n", liste, dict)

    # Suppression d'éléments
    liste.pop(0)
    dict.pop('age')

    print("Suppression d'éléments:\n", liste, dict)

    Valeurs initiales:
     [1, 'deux', 3] {'age': 77, 'nom': 'Doe', 'prenom': 'John'}
    Modification des valeurs par assignation:
     [1, 2, 3] {'age': 17, 'nom': 'Doe', 'prenom': 'John'}
    Ajout d'éléments:
     [1, 2, 3, 4] {'age': 17, 'nom': 'Doe', 'nationalité': 'française', 'prenom': 'John'}
    Suppression d'éléments:
     [2, 3, 4] {'nom': 'Doe', 'nationalité': 'française', 'prenom': 'John'}

Si on a besoin de modifier une liste ou un dictionnaire, mais que l'on veut garder une trace des objets initiaux, il faut commencer par en créer une copie, il ne suffit pas de créer une variable suplémentaire sans quoi cette variable serait elle aussi modifiée si l'objet initial changeait: l'assignation est dite par référence dans ce cas.

In [20]:

    # Valeurs initiales
    L = [ 1, 'deux' , 3]
    print("Valeurs initiales:\n", L)

    # Création d'une référencxe à la liste par simple assignation
    L_ref = L

    # Création d'une copie de la liste
    L_copie = list(L)

    # Modification de la liste initiale
    L[1] = 2

    print("Modification de la liste L:")
    print("La liste L a bien, été modifiée:", L)
    print("La liste L_ref a aussi été modifiée car il s'agit juste d'une référence vers la liste L:", L_ref)
    print("La copie L_copie n'a pas été modifiée:", L_copie)

    Valeurs initiales:
     [1, 'deux', 3]
    Modification de la liste L:
    La liste L a bien, été modifiée: [1, 2, 3]
    La liste L_ref a aussi été modifiée car il s'agit juste d'une référence vers la liste L: [1, 2, 3]
    La copie L_copie n'a pas été modifiée: [1, 'deux', 3]

# Entrée des données

Dans cette phase peut aussi figurer ce qu’on appelle l’entrée des données, qui peut se manifester par la saisie de caractères ou de nombres sur le clavier, ou la lecture de la position du pointeur de la souris, ou encore par la lecture d’un fichier contenant ces nombres ou caractères.

Il s’agit aussi de repérer les résultats intermédiaires qu’il est bon de mémoriser pour la suite car indispensables au traitement.

Il est parfois utile d’utiliser des variables auxiliaires pour ne pas perturber les données initiales.

## Entrée des données au clavier

In [21]:

    chiffre = input("Entrer la valeur du chiffre souhaité: ")

    Entrer la valeur du chiffre souhaité: 7

In [22]:

    print(chiffre)

    7

Attention cependant, cette valeur est une chaîne de caractère, et les opérations sont celles des `string`s.

In [23]:

    print(type(chiffre))
    chiffre*10

    <class 'str'>

Out[23]:

    '7777777777'

Il convient de la convertir un nombre entier ou flottant avant d'utiliser la valeur.

In [24]:

    chiffre = int(chiffre)
    10*chiffre

Out[24]:

    70

## Lecture d'un fichier

Voici par exemple comment ouvrir et lire un fichier appelé `lorem.txt` contenu dans le répértoire `data` du dossier courant.

In [27]:

    fichier = open('data/lorem.txt')
    fichier.read()

Out[27]:

    'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod\ntempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At\nvero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren,\nno sea takimata sanctus est Lorem ipsum dolor sit amet.\n'

Dans la sortie précédente, les caractères `\n` représentent les sauts de ligne, on peut aussi lire le fichier ligne par ligne.

In [29]:

    fichier = open('data/lorem.txt')
    fichier.readline()

Out[29]:

    'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod\n'